#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/SoDB.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/sensors/SoTimerSensor.h>

#define MAX_STRESS 10000000.0

int make_connection(char *, int);
void TimerCallback(void *Data, SoSensor *sensor);

SoTransform	*transformp;
SoTransform	*transformv;
SoMaterial	*material;
SoMaterial	*reflection;

float red[91];
float green[91];
float blue[91];

int descriptor;

int main(int argc, char **argv) {
	int i, port;
	float length;
	float scale;
	char server[100];
	FILE *fp;

	Widget window;
	SoSeparator *root;
	SoInput *SceneInput;
	SoXtExaminerViewer *viewer;
	SoTimerSensor *TimerSensor;

/*
 *      Parse command line arguments and create a socket
 *
 */

	for (i=1;i<argc - 1;i++) {
		if (strcmp(*(argv+i),"-s") == 0) {      // Server switch
			i++;
			strcpy(server,*(argv + i));     // Set server to the command line argument
		}
		if (strcmp(*(argv+i),"-p") == 0) {      // Port switch
			i++;
			port = atoi(*(argv + i));       // Set port to the command line argument
		}
	}

	if (!server || !port) {
		printf("Usage: %s -s server -p port\n",*argv);
		printf("\t-s server\tserver to connect to on specified port\n");
		printf("\t-p port\t\tport to listen for incomming clients\n");
		exit(0);
	}

	if ((descriptor = make_connection(server, port)) <= 0) {
		printf("Error:  Unable to connect to %s:%d\n",server,port);
		exit(0);
	}

/*
 *      Arguments Parsed, Socket Created, Code Continues...
 *
 */

	/* Read RGB color values (of the rainbow) from file */
	fp = fopen("COLOR_PATTERNS", "r");
	for (i=0;i<91;i++) fscanf(fp, "%f%f%f", &red[i], &green[i], &blue[i]);
	fclose(fp);

	/* Initialize Inventor and Xt; create new viewer */
	window = SoXt::init(*argv);
	viewer = new SoXtExaminerViewer(window);

	/* Parse the Inventor file */
	SceneInput = new SoInput;
	SceneInput->openFile("gview.iv");
	root = new SoSeparator;
	root = SoDB::readAll(SceneInput);
	SceneInput->closeFile();

	/* Create pointers to named nodes */
	transformp = (SoTransform *) SoNode::getByName("cylinder_transformp");
	transformv = (SoTransform *) SoNode::getByName("cylinder_transformv");
	material = (SoMaterial *) SoNode::getByName("material");
	reflection = (SoMaterial *) SoNode::getByName("reflection");
	/* We know the cylinder WAS made as length 50 with proe */
	/* So, first scale back to 1.0 and then scale up to the initial length */
	read(descriptor, (char*) &length, sizeof(float));
	scale = length / 50.0;
	
	transformp->scaleFactor.setValue(scale, 1, 1);	
	

	/* Display SceneGraph in viewer */
	viewer->setSceneGraph(root);
	viewer->show();
	
	/* Schedule invocation of a function with void* arguments */
	TimerSensor = new SoTimerSensor(TimerCallback, NULL);
	TimerSensor->schedule();

	/* Display viewer window and initialize event handler */
	SoXt::show(window);
	SoXt::mainLoop();
}

void TimerCallback(void *arguments, SoSensor *sensor) {
	int color, connection;
	float stress, strain;
	int index;

	/* Read values from Server */
	read(descriptor, (char*) &stress, sizeof(float));
	read(descriptor, (char*) &strain, sizeof(float));

	printf("I got stress = %f, and strain = %f\n", stress, strain);
	/* Modify scene with values from the server */
	transformv->scaleFactor.setValue((1.0 + strain), 1, 1);	
	index = (int) ((stress/MAX_STRESS) * 90.0 );
	printf("INDEX = %d\n", index);
	color = index;
	
	
	material->diffuseColor.setValue(red[color], green[color], blue[color]);

	/* Check if Labview has disconnected */
	read(descriptor, (char*) &connection, sizeof(int));
	write(descriptor, (char*) &connection, sizeof(int));

	if (connection == 0) {
		close(descriptor);
		exit(0);
	}

	/* Schedule another invocation of the function */
	sensor->schedule();
}
